package graphs;

import java.util.Iterator;
import java.util.List;

/**
 * Graph interface.
 *
 * @version 2.0.0 Dec 2001
 */
public interface Graph {
	
  /* 1.General methods - return global information about the graph */
  
  /** Returns the numers of vertices in this graph. */
  public int numVertices();
  
  /** Returns the number of edges in this graph.*/
  public int numEdges();

  /** Returns an iterator of the vertex positions in this graph. */  
  public Iterator vertices();

  /** Returns an iterator of the edge positions in this graph. */  
  public Iterator edges();
        
  
  
  /* Methods that take vertex and edge positions as arguments: */
  
  /** Returns an array of size 2 storing the end vertices of e. */
  public Vertex[] endVertices(Edge e);

  /** Returns the endpoint of edge e distinct from v. */  
  public Vertex opposite(Vertex v, Edge e);
  
  /** Returns whether vertices v and w are adjacent. */
  public boolean areAdgacent(Vertex v, Vertex w);
  
  
  /** Returns the degree of the vertex v: the number of all incident edges. */  
  public int degree(Vertex v);

  /** Returns an iterator of the vertices adjacent to v. */
  public Iterator adjacentVertices(Vertex v);

  /** Returns an iterator of the edges incident upon v. */  
  public Iterator incidentEdges(Vertex v);
  
  
  
}//to do: error conditions handling