package graphs.traversal;

import graphs.*;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * This class specializes DFS to determine if the connected component of
 * the start vertex contains a cycle and if so return it.
 * The algorithm returns an iterator of vertices in the cycle or an
 * empty if there is no such cycle.
 *
 * @version 1.1.0 Dec 2001
 */
public class FindCycleDFS extends DFS {
  
  protected ArrayList path;
  protected boolean done;
  protected Vertex cycleStart;
  
  /** Retuns an iterator of all visited vertices in the cicle */
  public Object execute(Graph graph, Vertex start, Object info) {
    super.execute(graph, start, info);
    path = new ArrayList();
    done = false;
    visit(start);  
    
    //copy the vertices u p to cycleStart from the path to the cycle
    ArrayList theCycle = new ArrayList();
    Iterator pathVerts = path.iterator();    
    while (pathVerts.hasNext()) {
      Vertex v = (Vertex) pathVerts.next();
      theCycle.add(0, v);
      if (v == cycleStart) break;
    }
    return theCycle.iterator();
  }
  
  protected void startVisit(Vertex v) {
    path.add(0, v);
  }
  
  protected void finishVisit(Vertex v) {
    if (!done) {
      path.remove(path.get(0));
    }
  }
    
  //When a back edge has been encountered, the graph has a cycle
  protected void traverseBack(Edge e, Vertex from) {
    Iterator pathVerts = path.iterator();
    cycleStart = graph.opposite(from, e);
    done = true;
  }
  
  protected boolean isDone() {
    return done;
  }
    
}