package graphs.traversal;

import graphs.*;
import java.util.ArrayList;

/**
 * This class specializes DFS to determine if there is a path from a
 * start vertex to a target vertex. It returns an enumeration of
 * vertices in the path, or an empty enumeration if such a path does not exists.
 * The algorithm performs a DFS beginning at the start vertex. 
 *
 * @version 1.1.0 Dec 2001
 */
 public class FindPathDFS extends DFS {
  
  protected ArrayList path;
  protected boolean done;
  protected Vertex targetVert;
  
  public Object execute(Graph graph, Vertex start, Object info) {
    super.execute(graph, start, info);
    path = new ArrayList();
    done = false;
    targetVert = (Vertex) info; //target vertex is stored in info parameter
    visit(start);
    return path.iterator();
  }
  
  protected void startVisit(Vertex v) {
    path.add(0, v); //insert it at the first postition in the list
    if (v == targetVert) {
      done = true;
    }
  }
  
  protected void finishVisit(Vertex v) {
    if (!done) {
      path.remove(path.get(0));
    }
  }
  
  protected boolean isDone() {
    return done;
  }  
  
}