package graphs.traversal;

import graphs.*;
import java.util.Hashtable;
import java.util.Iterator;

/** 
 * Abstract class representing the search in an undirected graph. 
 * @author V.B.
 * @version 2.0.0 Dec 2001
 */
public abstract class Traversal {
  
	protected Graph graph;
  protected Object visitResult;
  
  protected Hashtable markedVerts;
  protected Hashtable markedEdges;
	
  /** Traversal constructor */
  public Traversal() {
    markedVerts = new Hashtable();
    markedEdges = new Hashtable();
  }
  
  /** 
   * Executes the in the given graph starting from vertex 'start'.
   */
  public Object execute(Graph graph, Vertex start, Object info) {
    this.graph = graph;
    return null;
  }
  
  /**
   * Search starting at vertex v.
   */
  public abstract Object visit(Vertex v);
  
  /** Marks the visited vertex */
  protected void mark(Vertex v) {
    markedVerts.put(v, v); //use the vertex as both key and value
  }
   
  /** Marks the visited edge */
  protected void mark(Edge e) {
    markedEdges.put(e, e); //use the edge as both key and value
  }
  
  /** Checks whether this vertex is visited */
  protected boolean isMarked(Vertex v) {
    return markedVerts.contains(v);
  }
  
  /** Checks whether the edge is visited */
  protected boolean isMarked(Edge e) {
    return markedEdges.contains(e);
  }
  

  /** To stop the search before the whole graph is explored */
  protected boolean isDone() {
    return false;
  }
  
}